#include "viewer.h"
#include "object.h"
#include <iostream>
#include "main.h"


Viewer::Viewer(int width, int height) {
	device = irr::createDevice(irr::video::EDT_OPENGL, irr::core::dimension2d<irr::u32>(width, height), false, false, false);
	
	driver = device->getVideoDriver();
	smgr = device->getSceneManager();
	guienv = device->getGUIEnvironment();
	
	
	cameraFPS = smgr->addCameraSceneNodeFPS(0, 50, 0.005);
	cameraFPS->setPosition(vector3df(-1.5,1.5,-1.5));
	cameraView = smgr->addCameraSceneNode(0,vector3df(0,0,0),vector3df(0,0,0));
	device->getCursorControl()->setVisible(false);
	smgr->getActiveCamera()->setPosition(vector3df(0,0,0));
	smgr->getActiveCamera()->setRotation(vector3df(0,0,0));
	cameraView->bindTargetAndRotation(true);
	cameraView->setNearValue((irr::f32)0.1);
	//smgr->setAmbientLight(video::SColorf(0.1,0.1,0.1,1));	
	//ILightSceneNode* light1 = smgr->addLightSceneNode( 0, core::vector3df(0,10,0), video::SColorf(0.9f,0.9f,0.9f), 1.0f, 1 );
	
	IAnimatedMesh* base = smgr->addHillPlaneMesh("Plane", dimension2d<f32>(3.0f,3.0f), dimension2d<u32>(100,100), 0, 0, dimension2d<f32>(0,0), dimension2d<f32>(250,250)); 
	ISceneNode* node = smgr->addAnimatedMeshSceneNode(base);
	node->setMaterialTexture(0, driver->getTexture("../../irrlicht/media/rockwall.bmp")); //irrlicht/media/rockwall.bmp
	node->setMaterialFlag(video::EMF_LIGHTING, false);
	
	ISceneNode* node2 = smgr->addSkyDomeSceneNode(driver->getTexture("../../irrlicht/media/skydome.jpg")); //irrlicht/media/skydome.jpg
	node->setMaterialFlag(video::EMF_LIGHTING, false);
	font = device->getGUIEnvironment()->getBuiltInFont();
}


void Viewer::setRotation(const core::vector3df& rotation)
{
	cameraView->setRotation(rotation);
}

void Viewer::updateScene() {
	for(int i = 0; i < geoms.size(); i++) {
		ISceneNode* node = (ISceneNode*)dGeomGetData(geoms[i]);

		const dReal right2left[]={-1,0,0,0,0,0,1,0,0,-1,0,0};
		dVector3 position;
		dMultiply0(position, right2left, (dReal*)dGeomGetPosition(geoms[i]), 3, 3, 1);
		node->setPosition(vector3df(position[0], position[1], position[2]));		

		vector3df rot3;
		const dReal right2left2[]={-1,0,0,0,0,0,1,0,0,1,0,0};
		dQuaternion r, u;
		dRtoQ(right2left2, r);
		const dQuaternion s = {r[0], r[1], r[2], r[3]};
		dQuaternion result;
		dGeomGetQuaternion(geoms[i], result);
		const dQuaternion t = {result[0],result[1],result[2],result[3]};
		dQMultiply0(u,s,t);
		const quaternion q3(u[1],u[2],u[3],u[0]);

		q3.toEuler(rot3);
		vector3df degrees(-rot3.X*180/PI, -rot3.Y*180/PI, rot3.Z*180/PI);
		node->setRotation(degrees);	
	}

}

void Viewer::addObject(dGeomID g) {
	geoms.push_back(g);
}

IVideoDriver* Viewer::getVideoDriver() {
	return driver;
}

ISceneManager* Viewer::getSceneManager() {
	return smgr;
}

int Viewer::isFont()
{
	if(this->font) return 1;
	return 0;
}

void Viewer::setFont(gui::IGUIFont* font)
{
	this->font =font;
}

void Viewer::setVisibleDebug(IAnimatedMeshSceneNode *eq)
{
	eq->setDebugDataVisible((scene::E_DEBUG_SCENE_TYPE)(eq->isDebugDataVisible()^scene::EDS_BBOX));
}

void Viewer::setCameraViewParent(ISceneNode* newParent)
{
	this->cameraView->setParent(newParent);
}


void Viewer::setTarget(dReal x,dReal y,dReal z)
{
	cameraView->setTarget(vector3df(x,y,z));
}

const core::vector3df& Viewer::getRotation()
{
	return cameraView->getRotation();
}
	
bool Viewer::draw() {
	if(device->run()) {
		//cameraView->setTarget(vector3df(this->target[0],this->target[1],this->target[2]));
		updateScene();
		driver->setViewPort(rect<s32>(0,0,640*2,480));
		driver->beginScene(true, true, SColor(255,100,101,140));

		smgr->setActiveCamera(cameraView);	//draw camera viewpoint
		driver->setViewPort(rect<s32>(0,0,640,480));
		smgr->drawAll();
		smgr->setActiveCamera(cameraFPS);		//draw FPS camera
		driver->setViewPort(rect<s32>(640,0,640*2,480));
		smgr->drawAll();
		guienv->drawAll();
//		if (font) font->draw(L"Robot",
//                                        core::rect<s32>(130,10,300,50),
//                                        video::SColor(255,255,255,255));
		
		driver->endScene();
		
		return true;
	} else {
		device->drop();
		return false;
	}
}