#include "camera.h"
#include <ode/ode.h>
#include <math.h>
#include "main.h"



Camera::Camera(dWorldID world,dReal width,dReal depth, dReal height)
{
	this->width = width;
	this->depth = depth;
	this->height = height;
	body = dBodyCreate(world);
	geom = dCreateBox(0,width,depth,height);
	stepPosition = 0;
	dJointGroupID cameraJointGroup = dJointGroupCreate(0);
	lockJoint = dJointCreateFixed(world,cameraJointGroup);

/*
	dMass m;
	m.mass = 1;
	dMassSetSphere(&m, 1.0f, radius);
	dMassAdjust(&m, mass);
	dBodySetMass(body, &m);
*/
	dGeomSetBody(geom, body);
}

void Camera::addToViewer(Viewer& v)
{
	mesh = v.getSceneManager()->getMesh("../mesh/camera.3ds");
	node = v.getSceneManager()->addAnimatedMeshSceneNode(mesh);
	dGeomSetData(geom, node);
	
	dReal* position = (dReal*)dBodyGetPosition(getBody());
	node->setPosition(vector3df((irr::f32)0, (irr::f32)0, (irr::f32)0));
	node->setScale(vector3df(camera_h/50, camera_h/50, camera_h/50));

	
//	node->setRotation(vector3df(90, 0, 0));

	//node->setMaterialTexture(0, v.getVideoDriver()->getTexture("../../ode/drawstuff/textures/checkered.ppm"));
	//node->setMaterialFlag(video::EMF_LIGHTING, false);
	
	v.addObject(this->geom);

	
}

dReal *Camera::getViewAngle(void)
{
	return this->viewAngle;
}
void Camera::setViewAngle(dReal a[3])
{
	dMatrix3 R;
	dReal b[3];
	const dReal R2[9] = {0,-1,0,1,0,0,0,0,1};
	dMultiply0(b,R2,a,3,3,1);
	
	
	dRFrom2Axes(R,a[0],a[1],a[2],b[0],b[1],b[2]);
	
	dGeomSetRotation(geom,R);
	viewAngle[0]=a[0];
	viewAngle[1]=a[1];
	viewAngle[2]=a[2];
}

void Camera::setPosition(Robot *robot, dReal x, dReal y, dReal z)
{
	dMatrix3 R;
	dVector3 axis10 = {0, 0, 1, 1.0f};
	dVector3 axisZ;
	dMultiply0(axisZ, getRotation(), axis10, 3,3,1);
	
	const dReal *armEndPosition = robot->getEndPos();

	dBodySetPosition(body,x,y,z);


}

IAnimatedMeshSceneNode* Camera::getNode()
{
	return node;
}
/*
void Camera::updatePosition(Viewer *viewer, dReal x,dReal y, dReal z)
{
	dBodySetPosition(body,x,y,z);
	viewer->setCameraViewPos(x,y,z);
}
*/
void Camera::setScale(const core::vector3df& scale)
{
	node = this->node;
	node->setScale(scale);
}

void Camera::setFocusPoint(dReal point[3])
{
	this->focusPoint[0] = point[0];
	this->focusPoint[1] = point[1];
	this->focusPoint[2] = point[2];
}

void Camera::moveToN(dReal *node,Robot *robot)
{
	setPosition(robot,node[0],node[1],node[2]);
}

extern int checkCameraCollision(int collision);


int Camera::checkCollision()
{
	int collision = 0;
	//dSpaceCollide(space, &collision, &nearCallback);
	collision = checkCameraCollision(collision);
	return collision;
}


dReal Camera::getFocusPointAngle()
{
	dReal *angle = getViewAngle();
	
	dReal angleD[3];
	angleD[0]=angle[0];
	angleD[1]=angle[1];
	angleD[2]=angle[2];
	dReal vToPointD[3];
	dVector3 anglevector={angleD[0],angleD[1],angleD[2],0};
	const dReal *position = getPosition();
	dVector3 vToPoint={focusPoint[0]-position[0],focusPoint[1]-position[1],focusPoint[2]-position[2],0};
	
	dSafeNormalize3(anglevector);
	dSafeNormalize3(vToPoint);
	angleD[0]=anglevector[0];
	angleD[1]=anglevector[1];
	angleD[2]=anglevector[2];
	vToPointD[0]=vToPoint[0];
	vToPointD[1]=vToPoint[1];
	vToPointD[2]=vToPoint[2];
	dReal realAngle = dDot(angle,vToPointD,3);
	angleToPoint = acos(realAngle);
	return realAngle;
}

void Camera::attachCameraJoint()
{
	dJointAttach(lockJoint,getBody(),NULL);
	releaseCameraJoint();
}
void Camera::lockCameraJoint()
{
	dJointAttach(lockJoint,getBody(),NULL);
	dJointEnable(lockJoint);
}
void Camera::releaseCameraJoint()
{
	dJointDisable(lockJoint);
}
void Camera::createPath(Vbprm *vbprm, int nodeNumber, dReal end[3], int nodeNo, int path[])
{
	int node,startNode,endNode,nextNode;
	dReal startPos[3],err[3];
	startPos[0] = vbprm->G[nodeNo].V[0];
	startPos[1] = vbprm->G[nodeNo].V[1];
	startPos[2] = vbprm->G[nodeNo].V[2];
	//dReal *realPath = new dReal[vbprm->numberOfSteps];
//	int dist[52*120*60] = {Inf};
//	int *previous = new int[vbprm->numberOfSteps*3];
//	for( int i=0;i<52*120*60;i++) previous[i]=i;
//	int visited[52*60*120] = {0};
//	int queue[52*60*120] = {-1};
	
	for(int step = 0;step<vbprm->numberOfSteps;step++)
	{
		if(vbprm->G[nodeNo].stepCollision[step+1] !=0 ){
			for(int c = 0;c<52*60*120;c++){
				if(vbprm->G[c].stepCollision[step+1] == 0){
					err[0] = vbprm->G[nodeNo].V[0]-vbprm->G[c].V[0];
					err[1] = vbprm->G[nodeNo].V[1]-vbprm->G[c].V[1];
					err[2] = vbprm->G[nodeNo].V[2]-vbprm->G[c].V[2];
					dReal errLen = dLENGTH(err);
					if (step ==0){
						path[step+1]=c;
						break;
					}
					else if(  errLen < 0.15 )
					{
								path[step+1]=c;
								break;
	
					}
					
			
				}
			}
		}
		else {
			path[step+1] = nodeNo;
		}
		nodeNo = path[step+1];

			/*
			for(int next = 0;next<5*5*5-1;next++){
				nextNode = vbprm->G[nodeNo].E[next];
				if(vbprm->G[nextNode].stepCollision[step] == 0){
					path[step]=nextNode;
					break;
				}
			}
		} else {
			path[step]=nodeNo;
		}*/


	}

	/*
	for(int i=0;i<52*120*60;i++){
		if(vbprm->G[i].V[0]==start[0]&&vbprm->G[i].V[1]==start[1]&&vbprm->G[i].V[2]==start[2]){
			startNode = i;
		}
		if(vbprm->G[i].V[0]==end[0]&&vbprm->G[i].V[1]==end[1]&&vbprm->G[i].V[2]==end[2]){
			endNode = i;
		}
	}
	*/
	//dist[startNode]=0;
//	node = startNode;






/*
	dReal positionError[3];
	path[0]=start[0];
	path[1]=start[1];
	path[2]=start[2];

	
	for(int i = 3;i<vbprm->numberOfSteps*3;i+=3){
		positionError[0] = end[0]-path[i-3];
		positionError[1] = end[1]-path[i-2];
		positionError[2] = end[2]-path[i-1];
		if(!(positionError[0]<0.05&&positionError[0]>-0.05)){
			path[i+0] = path[i-3]+(positionError[0]/abs(positionError[0]))*0.022;

		}
		else path[i+0] = path[i-3];
		if(!(positionError[1]<0.05&&positionError[1]>-0.05)){
			path[i+1] = path[i-2]+(positionError[1]/abs(positionError[1]))*0.022;
		}
		else path[i+1] = path[i-2];
		if(!(positionError[2]<0.05&&positionError[2]>-0.05)){
			path[i+2] = path[i-1]+(positionError[2]/abs(positionError[2]))*0.022;
		}
		else path[i+2] = path[i-1];
	}
	*/
	//path = realPath;
	

}