#include "robot.h"
#include "vbprm.h"
#include <ode/ode.h>
#include <math.h>
#include "main.h"


Robot::Robot(dWorldID world)
{
	
//Denavit-Hartenberg matrix
	/*
	DH = {	{0.26, (dReal)-PI/2, 0.675, -1},	// Base wrt mast
		{0.68, 0, 0, -1},										// Link1 wrt base
		{0.035, (dReal)PI/2, 0.07, -1},					// Link2 wrt link1
		{0, (dReal)-PI/2, 0.67, -1},						// Wrist1 wrt link2
		{0, (dReal)PI/2, 0, -1},							// Wrist2 wrt wrist1
		{0, 0, 0.15, -1},										// Wrist3 wrt wrist2
		};
		*/
	DH[0][0]=bOffsetX;		// Base wrt mast
	DH[0][1]=bAngle;
	DH[0][2]=bOffsetZ;
	DH[0][3]=-1;
	DH[1][0]=l1OffsetZ;		// Link1 wrt base
	DH[1][1]=0;
	DH[1][2]=0;
	DH[1][3]=-1;
	DH[2][0]=l2OffsetX;		// Link2 wrt link1
	DH[2][1]=l2Angle;
	DH[2][2]=l2OffsetZ;
	DH[2][3]=-1;
	DH[3][0]=0;				// Wrist1 wrt link2
	DH[3][1]=w1Angle;
	DH[3][2]=w1OffsetZ;
	DH[3][3]=-1;
	DH[4][0]=0;				// Wrist2 wrt wrist1
	DH[4][1]=w2Angle;
	DH[4][2]=0;
	DH[4][3]=-1;
	DH[5][0]=0;				// Wrist3 wrt wrist2
	DH[5][1]=0;
	DH[5][2]=w3OffsetZ;
	DH[5][3]=-1;
				


	initBlackBaseRot = -PI/2;
	blackBasePos[0] = -0.472;
	blackBasePos[1] = 0.556;
	blackBasePos[2] = -0.378;
	
	coordinate_inversion_mask[0]=-1;
	coordinate_inversion_mask[1]=1;
	coordinate_inversion_mask[2]=1;
	coordinate_inversion_mask[3]=-1;
	coordinate_inversion_mask[4]=1;
	coordinate_inversion_mask[5]=-1;
	
	initialCoordinates[0]=90*toRad;
	initialCoordinates[1]=-90*toRad;
	initialCoordinates[2]=90*toRad;
	initialCoordinates[3]=0;
	initialCoordinates[4]=0;
	initialCoordinates[5]=0;
	Inertial_BlackBase[0][0]=cos(initBlackBaseRot);
	Inertial_BlackBase[0][1]=-sin(initBlackBaseRot);
	Inertial_BlackBase[0][2]=0;
	Inertial_BlackBase[0][3]=blackBasePos[0];
	Inertial_BlackBase[1][0]=sin(initBlackBaseRot);
	Inertial_BlackBase[1][1]=cos(initBlackBaseRot);
	Inertial_BlackBase[1][2]=0;
	Inertial_BlackBase[1][3]=blackBasePos[1];
	Inertial_BlackBase[2][0]=0;
	Inertial_BlackBase[2][1]=0;
	Inertial_BlackBase[2][2]=1;
	Inertial_BlackBase[2][3]=blackBasePos[2];
	Inertial_BlackBase[3][0]=0;
	Inertial_BlackBase[3][1]=0;
	Inertial_BlackBase[3][2]=0;
	Inertial_BlackBase[3][3]=1;
	

	jointCoordinates_offset[0] =0.0;
	jointCoordinates_offset[1]=0.0;
	jointCoordinates_offset[2]=PI/2;
	jointCoordinates_offset[3]=0.0;
	jointCoordinates_offset[4]=0.0;
	jointCoordinates_offset[5]=0.0;

	dJointGroupID robotJointGroup = dJointGroupCreate(0);
	blackBase = new Part(world,blackBase_w,blackBase_d,blackBase_h);
	blackBase->setPath("../mesh/fr_blackBase.3ds");

	blackBaseJoint = dJointCreateFixed(world,robotJointGroup);
	

	base = new Part(world,base_w,base_d,base_h);
	base->setPath("../mesh/fr_base.3ds");

	baseJoint = dJointCreateHinge(world,robotJointGroup);

	
	link1 = new Part(world,link1_w,link1_d,link1_h);
	link1->setPath("../mesh/fr_link1.3ds");
	
	link1Joint = dJointCreateHinge(world,robotJointGroup);

	link2 = new Part(world,link2_w,link2_d,link2_h);
	link2->setPath("../mesh/fr_link2.3ds");

	link2Joint = dJointCreateHinge(world,robotJointGroup);

	wrist1 = new Part(world,wrist1_w,wrist1_d,wrist1_h);
	wrist1->setPath("../mesh/fr_wrist1.3ds");

	wrist1Joint = dJointCreateHinge(world,robotJointGroup);

	wrist2 = new Part(world,wrist2_w,wrist2_d,wrist2_h);
	wrist2->setPath("../mesh/fr_wrist2.3ds");

	wrist2Joint = dJointCreateHinge(world,robotJointGroup);

	wrist3 = new Part(world,wrist3_w,wrist2_d,wrist2_h);
	wrist3->setPath("../mesh/fr_wrist3.3ds");

	wrist3Joint = dJointCreateHinge(world,robotJointGroup);

	tip = new Part(world,0.001,0.001,0.001);
	tip->setPath("");
	tipJoint = dJointCreateHinge(world,robotJointGroup);


	

}

void Robot::addToSpace(dSpaceID space)
{
	dSpaceAdd(space,blackBase->getGeom() );
	dSpaceAdd(space,base->getGeom());
	dSpaceAdd(space,link1->getGeom());
	dSpaceAdd(space,link2->getGeom());
	dSpaceAdd(space,wrist1->getGeom());
	dSpaceAdd(space,wrist2->getGeom());
	dSpaceAdd(space,wrist3->getGeom());
	dSpaceAdd(space,tip->getGeom());
}


void Robot::addToViewer(Viewer &v)
{
	blackBase->addToViewer(v);
	base->addToViewer(v);
	link1->addToViewer(v);
	link2->addToViewer(v);
	wrist1->addToViewer(v);
	wrist1->setScale(vector3df(wrist1_w, wrist1_w, wrist1_w));
	wrist2->addToViewer(v);
	wrist2->setScale(vector3df(wrist2_w/3, wrist2_w/3, wrist2_w/3));
	wrist3->addToViewer(v);
	wrist3->setScale(vector3df(wrist3_w/3,wrist3_w/3,wrist3_w/3));
	//tip->addToViewer(v);
	
	v.setVisibleDebug(wrist3->getNode());
	
}

void Robot::setPosition(dReal x, dReal y, dReal z)
{
	dMatrix3 R;

	//Attach joints
	dVector3 axis10 = {0, 0, 1, 1.0f};
	dVector3 axis20 = {0, 1, 0, 1.0f};
	dVector3 axis1, axis2, axis3, axis4, axis5,axis6;
	dMultiply0(axis1, base->getRotation(), axis10, 3,3,1);
	dMultiply0(axis2, base->getRotation(), axis20, 3,3,1);
	dMultiply0(axis3, link1->getRotation(),axis20, 3,3,1);
	dMultiply0(axis4, link2->getRotation(),axis10, 3,3,1);
	dMultiply0(axis5, wrist1->getRotation(),axis20,3,3,1);
	dMultiply0(axis6, wrist2->getRotation(),axis10,3,3,1);

	//blackBase
	dBodySetPosition( blackBase->getBody() , x, y, z+1);

	dJointAttach(blackBaseJoint,blackBase->getBody(),NULL);
	
	

	//base
	dRFromAxisAndAngle(R,1,0,0,-PI/2);
	const dReal *bbPos = dBodyGetPosition(blackBase->getBody());
	dBodySetRotation(base->getBody(),R);
	dBodySetPosition(base->getBody(),bbPos[0]+bOffsetX,bbPos[1],bbPos[2]+bOffsetZ);

	dJointAttach(baseJoint,blackBase->getBody(),base->getBody());
	const dReal *bPos = dBodyGetPosition (base->getBody());
	dJointSetHingeAnchor (baseJoint,bPos[0],bPos[1],bPos[2]);
	dJointSetHingeAxis(baseJoint,0,0,1);
	dJointSetHingeParam(baseJoint,dParamLoStop,-184*toRad);
	dJointSetHingeParam(baseJoint,dParamHiStop,184*toRad);
	


	//link1
	dRFromAxisAndAngle(R,0,1,0,PI/2);
	dMatrix3 R2 = {0,1,0,0,0,0,-1,0,-1,0,0,0};	//Rotate 90deg around y and x axis
	dBodySetRotation(link1->getBody(),R2);
	dBodySetPosition(link1->getBody(),bPos[0]+bOffsetX,bPos[1]+0.2,bPos[2]+bOffsetZ-0.10);

	dJointAttach(link1Joint,base->getBody(),link1->getBody());
	const dReal *l1Pos = dBodyGetPosition (link1->getBody());
	dJointSetHingeAnchor (link1Joint,l1Pos[0],l1Pos[1],l1Pos[2]-0.5);
	dJointSetHingeAxis(link1Joint,axis2[0],axis2[1],axis2[2]);
	dJointSetHingeParam(link1Joint,dParamLoStop,-154*toRad);		
	dJointSetHingeParam(link1Joint,dParamHiStop,34*toRad);	

	//link2

	dBodySetPosition(link2->getBody(),l1Pos[0],l1Pos[1]-0.2,l1Pos[2]+link1_w-0.6);

	dJointAttach(link2Joint,link1->getBody(),link2->getBody() );
	const dReal *l2Pos = dBodyGetPosition(link2->getBody());
	dJointSetHingeAnchor(link2Joint,l2Pos[0],l2Pos[1]+0.035,l2Pos[2]);
	dJointSetHingeAxis(link2Joint,axis3[0],axis3[1],axis3[2]);
	dJointSetHingeParam(link2Joint,dParamLoStop,-129*toRad);		
	dJointSetHingeParam(link2Joint,dParamHiStop,153*toRad);

	//wrist1
//	dMatrix3 R3 = {0,1,0,0,0,0,1,0,1,0,0,0};
	dRFromAxisAndAngle(R,-1,0,0,PI/2);
	dBodySetRotation(wrist1->getBody(),R);
	dBodySetPosition(wrist1->getBody(),l2Pos[0],l2Pos[1]+w1OffsetY,l2Pos[2]+w1OffsetZ);
	dJointAttach(wrist1Joint,link2->getBody(),wrist1->getBody());
	const dReal *w1Pos = dBodyGetPosition(wrist1->getBody());
	dJointSetHingeAnchor(wrist1Joint,w1Pos[0],w1Pos[1],w1Pos[2]);
	dJointSetHingeAxis(wrist1Joint,axis4[0],axis4[1],axis4[2]);
	dJointSetHingeParam(wrist1Joint,dParamLoStop,-185*toRad);
	dJointSetHingeParam(wrist1Joint,dParamHiStop,185*toRad);

	//wrist2
	dBodySetPosition(wrist2->getBody(),w1Pos[0],w1Pos[1],w1Pos[2]+w2OffsetZ);
	dJointAttach(wrist2Joint,wrist1->getBody(),wrist2->getBody());
	const dReal *w2Pos = dBodyGetPosition(wrist2->getBody());
	dJointSetHingeAnchor(wrist2Joint,w2Pos[0],w2Pos[1],w2Pos[2]);
	dJointSetHingeAxis(wrist2Joint,axis5[0],axis5[1],axis5[2]);
	dJointSetHingeParam(wrist2Joint,dParamLoStop,-70*toRad);	
	dJointSetHingeParam(wrist2Joint,dParamHiStop,70*toRad);

	//wrist3
	dBodySetPosition(wrist3->getBody(),w2Pos[0],w2Pos[1],w2Pos[2]+w3OffsetZ);
	dJointAttach(wrist3Joint,wrist2->getBody(),wrist3->getBody());
	const dReal *w3Pos = dBodyGetPosition(wrist3->getBody());
	dJointSetHingeAnchor(wrist3Joint,w3Pos[0],w3Pos[1],w3Pos[2]);
	dJointSetHingeAxis(wrist3Joint,axis6[0],axis6[1],axis6[2]);
	dJointSetHingeParam(wrist3Joint,dParamLoStop,-185*toRad);	
	dJointSetHingeParam(wrist3Joint,dParamHiStop,185*toRad);

	//tip
	dBodySetPosition(tip->getBody(),w3Pos[0],w3Pos[1],w3Pos[2]+tipOffset);
	dJointAttach(tipJoint,wrist3->getBody(),tip->getBody());
	const dReal *tipPos = dBodyGetPosition(tip->getBody());
	dJointSetHingeAnchor(tipJoint,tipPos[0],tipPos[1],tipPos[2]);
	dJointSetHingeParam(tipJoint,dParamLoStop,0);
	dJointSetHingeParam(tipJoint,dParamHiStop,0);
	
}

void Robot::setTorque( dReal t)
{
	dJointAddHingeTorque(wrist2Joint,t);
}


const dReal* Robot::getEndPos(void)
{
	return dBodyGetPosition(tip->getBody());
}

dReal Robot::getlink2JointAngle()
{
	return dJointGetHingeAngle(link2Joint);
}

void Robot::moveArm(dReal x, dReal y, dReal z)
{
	dVector3 axis10 = {0, 0, 1, 1.0f};
	dVector3 axisZ;
	dMultiply0(axisZ, tip->getRotation(),axis10,3,3,1);
	
	
//	dBodyGetRelPointPos 
	dBodySetPosition(tip->getBody(),x,y,z);
}

void Robot::updateObjectTransformations()
{
	// Updates the Denavit-Hartenberg parameters with the joint angles of the arm
	for(int i=0; i<numberOfDOFs; i++){
		DH[i][3] = jointCoordinates_current[i]*coordinate_inversion_mask[i] + jointCoordinates_offset[i];
	}
	

	
}




dReal *Robot::getJointCoordinates_current()
{
	return this->jointCoordinates_current;
}

void Robot::setJointCoordinates_current(dReal *jointCoordinates)
{
	for(int i = 0;i<numberOfDOFs;i++)
	{
		this->jointCoordinates_current[i]=jointCoordinates[i];
	}
	updateObjectTransformations();
}

dReal *Robot::getJointCoordinates_desired()
{
	return this->jointCoordinates_desired;
}

void Robot::setJointCoordinates_desired(dReal *jointCoordinates)
{
	for(int i=0;i<numberOfDOFs;i++)
	{
		this->jointCoordinates_desired[i]=jointCoordinates[i];
	}
}

void Robot::resetJointMovement()
{
	dJointSetHingeParam(baseJoint,dParamLoStop,-184*toRad);
	dJointSetHingeParam(baseJoint,dParamHiStop,184*toRad);
	dJointSetHingeParam(link1Joint,dParamLoStop,-154*toRad);		
	dJointSetHingeParam(link1Joint,dParamHiStop,34*toRad);	
	dJointSetHingeParam(link2Joint,dParamLoStop,-129*toRad);		
	dJointSetHingeParam(link2Joint,dParamHiStop,153*toRad);
	dJointSetHingeParam(wrist1Joint,dParamLoStop,-185*toRad);
	dJointSetHingeParam(wrist1Joint,dParamHiStop,185*toRad);
	dJointSetHingeParam(wrist2Joint,dParamLoStop,-70*toRad);	
	dJointSetHingeParam(wrist2Joint,dParamHiStop,70*toRad);
	dJointSetHingeParam(wrist3Joint,dParamLoStop,-185*toRad);	
	dJointSetHingeParam(wrist3Joint,dParamHiStop,185*toRad);
	dJointSetHingeParam(tipJoint,dParamLoStop,0);
	dJointSetHingeParam(tipJoint,dParamHiStop,0);
}

void Robot::stopJointMovement()
{
	dReal angle;
	angle = dJointGetHingeAngle(baseJoint);
	dJointSetHingeParam(baseJoint,dParamLoStop,angle);
	dJointSetHingeParam(baseJoint,dParamHiStop,angle);
	angle = dJointGetHingeAngle(link1Joint);
	dJointSetHingeParam(link1Joint,dParamLoStop,angle);		
	dJointSetHingeParam(link1Joint,dParamHiStop,angle);	
	angle = dJointGetHingeAngle(link2Joint);
	dJointSetHingeParam(link2Joint,dParamLoStop,angle);		
	dJointSetHingeParam(link2Joint,dParamHiStop,angle);
	angle = dJointGetHingeAngle(wrist1Joint);
	dJointSetHingeParam(wrist1Joint,dParamLoStop,angle);
	dJointSetHingeParam(wrist1Joint,dParamHiStop,angle);
	angle = dJointGetHingeAngle(wrist2Joint);
	dJointSetHingeParam(wrist2Joint,dParamLoStop,angle);	
	dJointSetHingeParam(wrist2Joint,dParamHiStop,angle);
	angle = dJointGetHingeAngle(wrist3Joint);
	dJointSetHingeParam(wrist3Joint,dParamLoStop,angle);	
	dJointSetHingeParam(wrist3Joint,dParamHiStop,angle);
	angle = dJointGetHingeAngle(tipJoint);
	dJointSetHingeParam(tipJoint,dParamLoStop,angle);
	dJointSetHingeParam(tipJoint,dParamHiStop,angle);
}


	

void Robot::moveInitPos(dReal endPos[3],dReal error,Viewer *viewer)
{
	resetJointMovement();
	dReal newX, newY, newZ;

	dReal positionError[3];

	const dReal *position;
	dJointAddHingeTorque(this->link2Joint, -1);
	dJointAddHingeTorque(this->link2Joint, 0);
	while(viewer->draw()){
	position = tip->getPosition();

	positionError[0] = endPos[0]-position[0];
	positionError[1] = endPos[1]-position[1];
	positionError[2] = endPos[2]-position[2];
	
	//error = dLENGTH(positionError);
	error = positionError[0];
	if(positionError[1]>error)
		error=positionError[1];
	if(positionError[2]>error)
		error=positionError[2];



	if(positionError[0]!=0){
		newX = position[0]+(positionError[0]/abs(positionError[0]))*moveStep;
	}
	else{
		newX = 0;
	}
	if(positionError[1]!=0){
		newY = position[1]+(positionError[1]/abs(positionError[1]))*moveStep;
	}else{
		newY = 0;
	}
	if(positionError[2]!=0){
		newZ = position[2]+(positionError[2]/abs(positionError[2]))*moveStep;
	}else{
		newZ = 0;
	}
	moveArm(newX,newY,newZ);
	const dReal *tipp=wrist2->getPosition();
	//dReal target[3] = {-tipp[0],-tipp[1],tipp[2]};
	viewer->target[0]=-tipp[0];
	viewer->target[1]=-tipp[1];
	viewer->target[2]=tipp[2];
	//viewer->setRotation(vector3df(-newX,-newY,newZ));
	//viewer->font->draw(L"Robot test text",
      //                                  core::rect<s32>(130,60,300,100),
        //                                video::SColor(255,255,255,255));

	ode_simulation();
	
	if (error <0.05) {
		stopJointMovement();
		break;
	}

	}
	

}

void Robot::moveInitPos2(dReal angle, Viewer *viewer)
{
	dJointAddHingeTorque(this->link2Joint, -1);

	while(viewer->draw()){
		if(this->getlink2JointAngle()<angle){
			dJointAddHingeTorque(this->link2Joint, 1);

			break;
		}
		ode_simulation();
	}

}

void Robot::moveToPos(dReal endPos[3],Viewer *viewer,Camera *camera,Vbprm *vbprm,int path[],dReal cameraDist[], dReal cameraAngle[])
{
	dReal newX, newY, newZ;
	dReal error = 0;
	dReal positionError[3];
	resetJointMovement();
	const dReal *position;

	while(viewer->draw()){
	position = tip->getPosition();

	positionError[0] = endPos[0]-position[0];
	positionError[1] = endPos[1]-position[1];
	positionError[2] = endPos[2]-position[2];
	
	//error = dLENGTH(positionError);
	error = positionError[0];
	if(positionError[1]>error)
		error=positionError[1];
	if(positionError[2]>error)
		error=positionError[2];



	if(positionError[0]!=0){
		newX = position[0]+(positionError[0]/abs(positionError[0]))*moveStep;
	}
	else{
		newX = 0;
	}
	if(positionError[1]!=0){
		newY = position[1]+(positionError[1]/abs(positionError[1]))*moveStep;
	}else{
		newY = 0;
	}
	if(positionError[2]!=0){
		newZ = position[2]+(positionError[2]/abs(positionError[2]))*moveStep;
	}else{
		newZ = 0;
	}
	
	//viewer->target[0]=-tipp[0];
	//viewer->target[1]=-tipp[1];
	//viewer->target[2]=tipp[2];

	
	moveArm(newX,newY,newZ);
	dReal pnt[3]={newX,newY,newZ};
	dReal tempDist[3],tempDistAbs,tempAngleVect[3],tempAngle, dotProduct,tempDist2;
	dReal tempVect[3] = {1,0,0};
	const dReal *tipp=wrist3->getPosition();
	if(camera->stepPosition<vbprm->numberOfSteps-1){
		//camera->setPosition(this,path[camera->stepPosition],path[camera->stepPosition+1],path[camera->stepPosition+2]);
		camera->stepPosition++;
		int nextStepNo = camera->stepPosition;
		int nextNodeNo = path[nextStepNo];
		if( (nextNodeNo >=0) && (nextNodeNo < 120*52*60) ){
			tempDist[0] = vbprm->G[nextNodeNo].V[0] - position[0];
			tempDist[1] = vbprm->G[nextNodeNo].V[1] - position[1];
			tempDist[2] = vbprm->G[nextNodeNo].V[2] - position[2];
			tempDistAbs = dLENGTH(tempDist);
			if(tempDistAbs<cameraDist[0]) cameraDist[0] = tempDistAbs;
			if(tempDistAbs>cameraDist[1]) cameraDist[1] = tempDistAbs;
			cameraDist[2] = cameraDist[2]+((tempDistAbs-cameraDist[2])/camera->stepPosition);
			
			dotProduct = (tempDist[0]*tempVect[0])+(tempDist[1]*tempVect[1])+(tempDist[2]*tempVect[2]);
			tempDist2 = dLENGTH(tempVect);
			tempAngle = acos(dotProduct/(tempDistAbs*tempDist2));
			if(tempAngle<cameraAngle[0]) cameraAngle[0] = tempAngle;
			if(tempAngle>cameraAngle[1]) cameraAngle[1] = tempAngle;
			cameraAngle[2] = cameraAngle[2]+((tempAngle-cameraAngle[2])/camera->stepPosition);

			//camera->releaseCameraJoint();
			camera->setPosition(this,vbprm->G[nextNodeNo].V[0],vbprm->G[nextNodeNo].V[1],vbprm->G[nextNodeNo].V[2]);
			//camera->lockCameraJoint();
		}
		
	}
	camera->setFocusPoint(pnt);
	dReal angleToPoint = camera->getFocusPointAngle();
	//camera->setViewAngle(angleToPoint)
	
	viewer->setTarget(-tipp[0],tipp[1]+2,tipp[2]);
	
	//dVector3 tipp;
	//dBodyCopyPosition(wrist3->getBody(),tipp);


	//int currentNode = vbprm->getCurrentNode();
	//int nextNode =vbprm->G[currentNode].E[0];
	//dReal *nextNodePos = vbprm->G[nextNode].V;
	//camera->moveToN(nextNodePos,this);
	//viewer->setRotation(vector3df(-newX,-newY,newZ));
	ode_simulation();
	if (error <0.05) {	
		stopJointMovement();
		break;
	}
	}
}


void Robot::simulateMoveToPos(dReal endPos[3],Viewer *viewer,Camera *camera,Vbprm *vbprm)
{
	dReal newX, newY, newZ;
	dReal error = 0;
	dReal positionError[3];
	resetJointMovement();
	const dReal *position;

	
	position = tip->getPosition();

	positionError[0] = endPos[0]-position[0];
	positionError[1] = endPos[1]-position[1];
	positionError[2] = endPos[2]-position[2];
	
	//error = dLENGTH(positionError);
	error = positionError[0];
	if(positionError[1]>error)
		error=positionError[1];
	if(positionError[2]>error)
		error=positionError[2];



	if(positionError[0]!=0){
		newX = position[0]+(positionError[0]/abs(positionError[0]))*moveStep;
	}
	else{
		newX = 0;
	}
	if(positionError[1]!=0){
		newY = position[1]+(positionError[1]/abs(positionError[1]))*moveStep;
	}else{
		newY = 0;
	}
	if(positionError[2]!=0){
		newZ = position[2]+(positionError[2]/abs(positionError[2]))*moveStep;
	}else{
		newZ = 0;
	}
	moveArm(newX,newY,newZ);
	dReal pnt[3]={newX,newY,newZ};
	
	camera->setFocusPoint(pnt);
	dReal angleToPoint = camera->getFocusPointAngle();

	const dReal *tipp=wrist2->getPosition();
	viewer->setTarget(-tipp[0],-tipp[1],tipp[2]+0.3);
	//int currentNode = vbprm->getCurrentNode();
	//int nextNode =vbprm->G[currentNode].E[0];
	//dReal *nextNodePos = vbprm->G[nextNode].V;
	//camera->moveToN(nextNodePos,this);
	//viewer->setRotation(vector3df(-newX,-newY,newZ));
	ode_simulation();
	
	
}