#ifndef ROBOT_H
#define ROBOT_H

#include "object.h"
#include "Part.h"
#include "camera.h"
#include "vbprm.h"


#define NUM_PARTS 7
#define DOF 6

/*

dJointGroupID  robotJointGroup;
IAnimatedMeshSceneNode* blackBaseNode;


dGeomID groundGeom;


dGeomID blackBaseGeom;
dGeomID baseGeom;
dGeomID link1Geom;
dGeomID link2Geom;
dGeomID wrist1Geom;
dGeomID wrist2Geom;
dGeomID wrist3Geom;

dBodyID blackBaseBody;
dBodyID baseBody;
dBodyID link1Body;
dBodyID link2Body;
dBodyID wrist1Body;
dBodyID wrist2Body;
dBodyID wrist3Body;

dJointID groundJoint;
dJointID blackBaseJoint;
dJointID baseLink1Joint;
dJointID link1Link2Joint;
dJointID link2Wrist1Joint;
dJointID wrist1Wrist2Joint;
dJointID wrist2Wrist3Joint;
*/

#define blackBase_w	1.0		//x
#define blackBase_h	0.58	//y
#define blackBase_d	0.783	//z
#define base_w		1.0		//x
#define base_d		0.811	//y
#define base_h		0.697	//z
#define link1_w		1.0		//x
#define link1_h		0.231	//z
#define link1_d		0.254	//y
#define link2_w		0.258	//y
#define link2_d		0.379	//x
#define link2_h		0.741	//z
#define wrist1_h	1.0		//y
#define wrist1_w	0.466	//x
#define wrist1_d	0.465	//z
#define wrist2_w	1.0		//z
#define wrist2_h	0.969	//y
#define wrist2_d	0.581	//x
#define wrist3_h	0.57	//y
#define wrist3_d	0.92
#define wrist3_w	0.92

//#define PI 3.1415
#define toRad (double)PI/180
#define toDeg = (double)180/PI

#define bOffsetX 0.26
#define bOffsetZ 0.675
#define bAngle -PI/2
#define l1OffsetZ 0.68
#define l2OffsetX 0.035
#define l2OffsetZ 0.07
#define l2Angle PI/2
#define w1OffsetY 0.00
#define w1OffsetZ 0.67
#define w1Angle -PI/2
#define w2OffsetZ 0.2
#define w2Angle PI/2
#define w3OffsetZ 0.15
#define tipOffset 0.1

#define moveStep 0.03;

#define numberOfDOFs 6

void ode_simulation(void);
 
class Camera;
class Vbprm;

class Robot:public Object
{
	protected:
		Part *blackBase,*base,*link1,*link2,*wrist1,*wrist2, *wrist3,*tip;
		dJointID blackBaseJoint,baseJoint,link1Joint,link2Joint,wrist1Joint,wrist2Joint,wrist3Joint,tipJoint;
		dReal initBlackBaseRot;
		dReal blackBasePos[3];
		dReal Inertial_BlackBase[4][4];
		dReal jointCoordinates_current[numberOfDOFs];
		dReal jointCoordinates_desired[numberOfDOFs];
		dReal jointCoordinates_offset[numberOfDOFs];
		dReal coordinate_inversion_mask[numberOfDOFs];
		dReal initialCoordinates[numberOfDOFs];
		dReal DH[6][4];


	public:
		Robot(dWorldID);
	void setPosition(dReal x, dReal y,dReal z);
	void addToSpace(dSpaceID space);
	void addToViewer(Viewer& v);
	void setTorque(dReal t);
	void moveArm(dReal x, dReal y, dReal z);
	void setEndPos(void);
	const dReal* getEndPos(void);
	dReal Robot::moveInitPos(dReal endPos[3],dReal error);
	void updateObjectTransformations();
	void setJointCoordinates_desired(dReal *jointCoordinates);
	dReal *getJointCoordinates_desired(void); 
	void setJointCoordinates_current(dReal *jointCoordinates);
	dReal *getJointCoordinates_current(void);
	dReal *calculateInverseKinematics(dReal *p_39, dReal R_3_9);
	dReal getlink2JointAngle();
	void moveInitPos(dReal initPos[3],dReal error, Viewer *viewer);
	void Robot::moveInitPos2(dReal angle, Viewer *viewer);
	void moveToPos(dReal position[3],Viewer *viewer, Camera *camera,Vbprm *vbprm,int path[], dReal cameraDist[], dReal cameraAngle[]);
	void simulateMoveToPos(dReal endPos[3],Viewer *viewer,Camera *camera,Vbprm *vbprm);
	void Robot::resetJointMovement();
	void Robot::stopJointMovement();
	void updateCameraPosition();
};


#endif